/* RAM Leakage Detector */

#include <tos.h>
#include "ram_chck.h"

typedef struct {
	long id;
	long* start;
	long*	end;
 } LIST;

static struct {
	long magic;
	LIST *list;
	int num;
	int change;
 } *ram;

static RAM_CHECK ret;

/* *******************************************************************
                          cookie_jar
Returns address of cookie jar, or NULL.
Must be executed in supervisor mode (Supexec).
******************************************************************* */
static long cookie_jar(void)
 {
	long *p;

	p = (long*) 0x5A0L;
	return(*p);
 }

/* *******************************************************************
                          get_cookie
pname is address of cookie name (long or 4 character string).
pval returns the cookie value (long or bit field structure).

Returns TRUE (-1) if cookie found, FALSE (0) if not found.
******************************************************************* */
static int get_cookie(void* pname, void* pval)
 {
	char *p;
	long name,*jar;

	jar = (long*) Supexec(cookie_jar);
	if(!jar) return(0);

  p = pname;
  name = ((((( (long) p[0] << 8) | p[1]) << 8) | p[2]) << 8) | p[3];

	while(*jar) {
		if(*jar == name) {
			*((long*) pval) = *(++jar);
			return(-1);
		 }
		jar+=2;
	 }
	return(0);
 }

/* *****************************************************************
                            ram_check
Returns NULL normally, or a structure pointer if a ram leakage is
detected.
***************************************************************** */

RAM_CHECK *ram_check(void)
 {
	LIST *test;
	int count;
	long *p;

	if(!get_cookie("LEAK",&ram)) return(0L);

	test = ram->list;
	p = 0L;
	for(count=0; count < ram->num; count++, test++) if(test->end) {
		if(*(test->start) != ram->magic) {
			p = test->start;
			break;
		 }
		if(*(test->end) != ram->magic) {
			p = test->end;
			break;
		 }
	 }
	if(!p) return(0L);

	ret.block = test->start + 1;
	ret.test = p;
	ret.magic = ram->magic;
	return(&ret);
 }

/* *****************************************************************
                            ram_reset
Causes last detected ram leak to no longer register.
Method is determined by code:
  0 = remove block from list.
  1 = restore magic number.
***************************************************************** */

void ram_reset(int code)
 {
	LIST *test;
	int count;
	long *p;

	if(!get_cookie("LEAK",&ram)) return;

	test = ram->list;
	p = 0L;
	for(count=0; count < ram->num; count++, test++) if(test->end) {
		if(*(test->start) != ram->magic) {
			p = test->start;
			break;
		 }
		if(*(test->end) != ram->magic) {
			p = test->end;
			break;
		 }
	 }

	if(p) switch(code) {
	 case 0:
		test->end = 0;
		break;
	 case 1:
		*p = ram->magic;
		break;
	 }
	return;
 }
